package net.wanji.opt.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.dto.CrossIdDTO;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.opt.dto.DiagnoCrossListDTO;
import net.wanji.opt.dto.strategy.AddOrUpdateSceneDTO;
import net.wanji.opt.service.DiagnoService;
import net.wanji.opt.vo.CrossIdAndLocationVO;
import net.wanji.opt.vo.DiagnoCrossVO;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.ws.rs.core.MediaType;
import java.util.List;

/**
 * 诊断优化
 *
 * @author Kent HAN
 */
@Api(value = "DiagnoController", description = "诊断优化")
@RequestMapping("/diagno")
@RestController
public class DiagnoController {

    private final DiagnoService diagnoService;

    public DiagnoController(DiagnoService diagnoService) {
        this.diagnoService = diagnoService;
    }

    @ApiOperation(value = "路口列表", notes = "路口列表", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/diagnoCrossList",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = DiagnoCrossVO.class),
    })
    public JsonViewObject diagnoCrossList(@RequestBody @Valid DiagnoCrossListDTO diagnoCrossListDTO) {
        List<DiagnoCrossVO> diagnoCrossVOList = diagnoService.diagnoCrossList(diagnoCrossListDTO);
        return JsonViewObject.newInstance().success(diagnoCrossVOList);
    }

    @ApiOperation(value = "查询路口当前场景", notes = "查询路口当前场景", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/queryCrossScene",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject queryCrossScene(@RequestBody CrossIdDTO crossIdDTO) {
        AddOrUpdateSceneDTO addOrUpdateSceneDTO = diagnoService.queryCrossScene(crossIdDTO);
        return JsonViewObject.newInstance().success(addOrUpdateSceneDTO);
    }

    @ApiOperation(value = "查询当前路口相邻路口", notes = "查询当前路口相邻路口", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/queryNeighborCross",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossIdAndLocationVO.class),
    })
    public JsonViewObject queryNeighborCross(@RequestBody CrossIdDTO crossIdDTO) {
        List<CrossIdAndLocationVO> crossIdAndLocationVOList = diagnoService.queryNeighborCross(crossIdDTO);
        return JsonViewObject.newInstance().success(crossIdAndLocationVOList);
    }
}