package net.wanji.opt.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.dto.CrossIdDTO;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.dao.entity.RidInfoEntity;
import net.wanji.databus.dao.mapper.RidInfoMapper;
import net.wanji.opt.dao.mapper.CrossDirInfoMapper;
import net.wanji.opt.dao.mapper.CrossInfoMapper;
import net.wanji.opt.dao.mapper.strategy.SceneMapper;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyMapper;
import net.wanji.opt.dao.mapper.trend.CrossDataRealtimeMapper;
import net.wanji.opt.dto.DiagnoCrossListDTO;
import net.wanji.opt.dto.strategy.AddOrUpdateSceneDTO;
import net.wanji.opt.dto.strategy.QuerySceneDTO;
import net.wanji.opt.po.base.CrossInfoPO;
import net.wanji.opt.po.strategy.ScenePO;
import net.wanji.opt.po.trend.CrossDataRealtimePO;
import net.wanji.opt.service.DiagnoService;
import net.wanji.opt.service.strategy.SceneService;
import net.wanji.opt.vo.CrossIdAndLocationVO;
import net.wanji.opt.vo.DiagnoCrossVO;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author Kent HAN
 * @date 2023/3/2 10:09
 */
@Service
@Slf4j
public class DiagnoServiceImpl implements DiagnoService {

    private final CrossInfoMapper crossInfoMapper;
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;
    private final SceneService sceneService;
    private final SceneMapper sceneMapper;
    private final CrossDirInfoMapper crossDirInfoMapper;
    private final RidInfoMapper ridInfoMapper;

    public DiagnoServiceImpl(CrossInfoMapper crossInfoMapper, CrossDataRealtimeMapper crossDataRealtimeMapper,
                             SceneService sceneService, SceneStrategyMapper sceneStrategyMapper, SceneMapper sceneMapper, CrossDirInfoMapper crossDirInfoMapper, RidInfoMapper ridInfoMapper) {
        this.crossInfoMapper = crossInfoMapper;
        this.crossDataRealtimeMapper = crossDataRealtimeMapper;
        this.sceneService = sceneService;
        this.sceneMapper = sceneMapper;
        this.crossDirInfoMapper = crossDirInfoMapper;
        this.ridInfoMapper = ridInfoMapper;
    }

    @Override
    public List<DiagnoCrossVO> diagnoCrossList(DiagnoCrossListDTO diagnoCrossListDTO) {
        String name = diagnoCrossListDTO.getName();
        Integer target = diagnoCrossListDTO.getTarget();
        if (target == 1) { // 路口
            return queryCross(name);
        }
        return null;
    }

    @Override
    public AddOrUpdateSceneDTO queryCrossScene(CrossIdDTO crossIdDTO) {
        // 查询路口场景代码
        String crossId = crossIdDTO.getCrossId();
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
        Integer status = crossDataRealtimePO.getStatus();
        // 获取场景名称
        ScenePO scenePO = sceneMapper.selectBySceneNum(status);
        String sceneName = scenePO.getSceneName();
        // 构造请求体
        QuerySceneDTO querySceneDTO = new QuerySceneDTO();
        querySceneDTO.setSceneName(sceneName);
        querySceneDTO.setSceneTarget(1);
        querySceneDTO.setPageNum(1);
        querySceneDTO.setPageSize(999);
        PageInfo<AddOrUpdateSceneDTO> pageInfo = sceneService.queryScene(querySceneDTO);
        List<AddOrUpdateSceneDTO> list = pageInfo.getList();
        AddOrUpdateSceneDTO addOrUpdateSceneDTO = list.get(0);

        return addOrUpdateSceneDTO;
    }

    @Override
    public List<CrossIdAndLocationVO> queryNeighborCross(CrossIdDTO crossIdDTO) {
        List<CrossIdAndLocationVO> crossIdAndLocationVOList = new ArrayList<>();
        String crossId = crossIdDTO.getCrossId();
        List<Integer> inDirs = crossDirInfoMapper.selectInDirsByCrossId(crossId);
        for (Integer inDir : inDirs) {
            CrossIdAndLocationVO crossIdAndLocationVO = new CrossIdAndLocationVO();
            RidInfoEntity ridInfo = ridInfoMapper.selectByEndInDir(crossId, inDir);
            if (ridInfo != null) {
                // 关联路口坐标
                String startCrossId = ridInfo.getStartCrossId();
                crossIdAndLocationVO.setCrossId(startCrossId);
                CrossInfoPO crossInfoPO = crossInfoMapper.selectById(startCrossId);
                if (crossInfoPO != null) {
                    String startLocationStr = crossInfoPO.getLocation();
                    double[] startLonLat = CrossUtil.getLonLat(startLocationStr);
                    List<Double> startLocation = new ArrayList<>();
                    startLocation.add(startLonLat[0]);
                    startLocation.add(startLonLat[1]);
                    crossIdAndLocationVO.setLocation(startLocation);
                }
            }
            crossIdAndLocationVOList.add(crossIdAndLocationVO);
        }
        return crossIdAndLocationVOList;
    }

    private List<DiagnoCrossVO> queryCross(String name) {
        List<DiagnoCrossVO> diagnoCrossVOList = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(name)) {
            // 根据name做模糊查询
            List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectByName(name);
            diagnoCrossVOList = getDiagnoCrossVOList(crossInfoPOList);
        } else {
            // 返回全量数据
            List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectAll();
            diagnoCrossVOList = getDiagnoCrossVOList(crossInfoPOList);
        }
        return diagnoCrossVOList;
    }

    private List<DiagnoCrossVO> getDiagnoCrossVOList(List<CrossInfoPO> crossInfoPOList) {
        List<DiagnoCrossVO> diagnoCrossVOList = new ArrayList<>();
        for (CrossInfoPO crossInfoPO : crossInfoPOList) {
            DiagnoCrossVO diagnoCrossVO = new DiagnoCrossVO();
            diagnoCrossVO.setCrossName(crossInfoPO.getName());
            String crossId = crossInfoPO.getId();
            diagnoCrossVO.setCrossId(crossId);
            CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
            Integer status = crossDataRealtimePO.getStatus();
            diagnoCrossVO.setScene(status);
            // 选择优先级最高的策略
            QuerySceneDTO querySceneDTO = new QuerySceneDTO();
            querySceneDTO.setSceneTarget(1);
            querySceneDTO.setPageNum(1);
            querySceneDTO.setPageSize(999);
            PageInfo<AddOrUpdateSceneDTO> sceneList = sceneService.queryScene(querySceneDTO);
            List<AddOrUpdateSceneDTO> list = sceneList.getList();
            for (AddOrUpdateSceneDTO addOrUpdateSceneDTO : list) {
                Integer sceneNum = addOrUpdateSceneDTO.getSceneNum();
                if (Objects.equals(sceneNum, status)) {
                    List<AddOrUpdateSceneDTO.StrategyListElement> strategyList =
                            addOrUpdateSceneDTO.getStrategyList();
                    AddOrUpdateSceneDTO.StrategyListElement strategyListElement = strategyList.get(0);
                    diagnoCrossVO.setStrategy(strategyListElement.getStrategyName());
                    // 选择优先级最高的方法
                    List<AddOrUpdateSceneDTO.IdeaListElement> ideaList = strategyListElement.getIdeaList();
                    AddOrUpdateSceneDTO.IdeaListElement ideaListElement = ideaList.get(0);
                    diagnoCrossVO.setIdea(ideaListElement.getIdeaName());
                }
            }
            diagnoCrossVOList.add(diagnoCrossVO);
        }
        return diagnoCrossVOList;
    }
}
